@@ -14,14 +14,17 @@ use storage_proofs_post::fallback::{
 };
 
 use crate::{
-    api::{as_safe_commitment, get_partitions_for_window_post, partition_vanilla_proofs},
+    api::{
+        as_safe_commitment, get_partitions_for_window_post, partition_vanilla_proofs,
+        single_partition_vanilla_proofs,
+    },
     caches::{get_post_params, get_post_verifying_key},
     parameters::window_post_setup_params,
     types::{
         ChallengeSeed, FallbackPoStSectorProof, PoStConfig, PrivateReplicaInfo, ProverId,
         PublicReplicaInfo, SnarkProof,
     },
-    PoStType,
+    PartitionSnarkProof, PoStType,
 };
 
 /// Generates a Window proof-of-spacetime with provided vanilla proofs.
@@ -42,8 +45,8 @@ pub fn generate_window_post_with_vanilla<Tree: 'static + MerkleTreeTrait>(
     let prover_id_safe: <Tree::Hasher as Hasher>::Domain =
         as_safe_commitment(&prover_id, "prover_id")?;
 
-    let vanilla_params = window_post_setup_params(&post_config);
-    let partitions = get_partitions_for_window_post(vanilla_proofs.len(), &post_config);
+    let vanilla_params = window_post_setup_params(post_config);
+    let partitions = get_partitions_for_window_post(vanilla_proofs.len(), post_config);
 
     let setup_params = compound_proof::SetupParams {
         vanilla_params,
@@ -55,7 +58,7 @@ pub fn generate_window_post_with_vanilla<Tree: 'static + MerkleTreeTrait>(
 
     let pub_params: compound_proof::PublicParams<'_, FallbackPoSt<'_, Tree>> =
         FallbackPoStCompound::setup(&setup_params)?;
-    let groth_params = get_post_params::<Tree>(&post_config)?;
+    let groth_params = get_post_params::<Tree>(post_config)?;
 
     let mut pub_sectors = Vec::with_capacity(vanilla_proofs.len());
     for vanilla_proof in &vanilla_proofs {
@@ -73,7 +76,7 @@ pub fn generate_window_post_with_vanilla<Tree: 'static + MerkleTreeTrait>(
     };
 
     let partitioned_proofs = partition_vanilla_proofs(
-        &post_config,
+        post_config,
         &pub_params.vanilla_params,
         &pub_inputs,
         partitions,
@@ -108,8 +111,8 @@ pub fn generate_window_post<Tree: 'static + MerkleTreeTrait>(
     let randomness_safe = as_safe_commitment(randomness, "randomness")?;
     let prover_id_safe = as_safe_commitment(&prover_id, "prover_id")?;
 
-    let vanilla_params = window_post_setup_params(&post_config);
-    let partitions = get_partitions_for_window_post(replicas.len(), &post_config);
+    let vanilla_params = window_post_setup_params(post_config);
+    let partitions = get_partitions_for_window_post(replicas.len(), post_config);
 
     let sector_count = vanilla_params.sector_count;
     let setup_params = compound_proof::SetupParams {
@@ -120,7 +123,7 @@ pub fn generate_window_post<Tree: 'static + MerkleTreeTrait>(
 
     let pub_params: compound_proof::PublicParams<'_, FallbackPoSt<'_, Tree>> =
         FallbackPoStCompound::setup(&setup_params)?;
-    let groth_params = get_post_params::<Tree>(&post_config)?;
+    let groth_params = get_post_params::<Tree>(post_config)?;
 
     let trees: Vec<_> = replicas
         .iter()
@@ -190,8 +193,8 @@ pub fn verify_window_post<Tree: 'static + MerkleTreeTrait>(
     let randomness_safe = as_safe_commitment(randomness, "randomness")?;
     let prover_id_safe = as_safe_commitment(&prover_id, "prover_id")?;
 
-    let vanilla_params = window_post_setup_params(&post_config);
-    let partitions = get_partitions_for_window_post(replicas.len(), &post_config);
+    let vanilla_params = window_post_setup_params(post_config);
+    let partitions = get_partitions_for_window_post(replicas.len(), post_config);
 
     let setup_params = compound_proof::SetupParams {
         vanilla_params,
@@ -222,7 +225,7 @@ pub fn verify_window_post<Tree: 'static + MerkleTreeTrait>(
     };
 
     let is_valid = {
-        let verifying_key = get_post_verifying_key::<Tree>(&post_config)?;
+        let verifying_key = get_post_verifying_key::<Tree>(post_config)?;
         let multi_proof = MultiProof::new_from_reader(partitions, proof, &verifying_key)?;
 
         FallbackPoStCompound::verify(
@@ -242,3 +245,69 @@ pub fn verify_window_post<Tree: 'static + MerkleTreeTrait>(
 
     Ok(true)
 }
+
+/// Generates a Window proof-of-spacetime with provided vanilla proofs of a single partition.
+pub fn generate_single_window_post_with_vanilla<Tree: 'static + MerkleTreeTrait>(
+    post_config: &PoStConfig,
+    randomness: &ChallengeSeed,
+    prover_id: ProverId,
+    vanilla_proofs: Vec<FallbackPoStSectorProof<Tree>>,
+    partition_index: usize,
+) -> Result<PartitionSnarkProof> {
+    info!("generate_single_window_post_with_vanilla:start");
+    ensure!(
+        post_config.typ == PoStType::Window,
+        "invalid post config type"
+    );
+
+    let randomness_safe: <Tree::Hasher as Hasher>::Domain =
+        as_safe_commitment(randomness, "randomness")?;
+    let prover_id_safe: <Tree::Hasher as Hasher>::Domain =
+        as_safe_commitment(&prover_id, "prover_id")?;
+
+    let vanilla_params = window_post_setup_params(post_config);
+    let partitions = get_partitions_for_window_post(vanilla_proofs.len(), post_config);
+
+    let setup_params = compound_proof::SetupParams {
+        vanilla_params,
+        partitions,
+        priority: post_config.priority,
+    };
+
+    let pub_params: compound_proof::PublicParams<'_, FallbackPoSt<'_, Tree>> =
+        FallbackPoStCompound::setup(&setup_params)?;
+    let groth_params = get_post_params::<Tree>(post_config)?;
+
+    let mut pub_sectors = Vec::with_capacity(vanilla_proofs.len());
+    for vanilla_proof in &vanilla_proofs {
+        pub_sectors.push(PublicSector {
+            id: vanilla_proof.sector_id,
+            comm_r: vanilla_proof.comm_r,
+        });
+    }
+
+    let pub_inputs = fallback::PublicInputs {
+        randomness: randomness_safe,
+        prover_id: prover_id_safe,
+        sectors: pub_sectors,
+        k: Some(partition_index),
+    };
+
+    let partitioned_proofs = single_partition_vanilla_proofs(
+        post_config,
+        &pub_params.vanilla_params,
+        &pub_inputs,
+        &vanilla_proofs,
+    )?;
+
+    let proof = FallbackPoStCompound::prove_with_vanilla(
+        &pub_params,
+        &pub_inputs,
+        vec![partitioned_proofs],
+        &groth_params,
+    )?;
+
+    info!("generate_single_window_post_with_vanilla:finish");
+
+    proof.to_vec().map(PartitionSnarkProof)
+}
